from netCDF4 import Dataset
from scipy import stats as sts
from pandas import DataFrame

import numpy as np
import os
import datetime
import matplotlib.pyplot as plt
from random import randint
from scipy.stats.stats import pearsonr
from scipy.stats import linregress

def remove_outliers(obs,mods):
    nobs=[]
    nmods=[]
    m=np.median(obs)
    q1,q3=np.percentile(obs,[25,75])
    iqr=q3-q1
    lb=q1-(1.5*iqr)
    ub=q3+(1.5*iqr)
    #outliers=np.zeros(len(lst))
    for i in range(len(obs)):
        if obs[i]>lb and obs[i]<ub and obs[i]>=0:
            nobs.append(obs[i])
            nmods.append(mods[i])
    return nobs,nmods

def pick_randoms(lst1,lst2,n):
    nlst1=[]
    nlst2=[]
    for i in range(n):
        a=randint(0,len(lst1)-1)
        nlst1.append(lst1[a])
        nlst2.append(lst2[a])
    return nlst1,nlst2
# EXTRACTING THE MODEL DATA AND CREATING BBP700 AND CHLA

model='/esarchive/scratch/mgalitap/aumont/pisces-v2-aumont17/PISCES_1m_19600101_19601231_ptrc_T.nc'
modeldata=Dataset(model,'r')

POC=np.array(modeldata.variables['POC'])
GOC=np.array(modeldata.variables['GOC'])
ZOO=np.array(modeldata.variables['ZOO'])
ZOO2=np.array(modeldata.variables['ZOO2'])
PHY=np.array(modeldata.variables['PHY'])
PHY2=np.array(modeldata.variables['PHY2'])

BBP700=POC+GOC+ZOO+ZOO2+PHY+PHY2

NCHL=np.array(modeldata.variables['NCHL'])
DCHL=np.array(modeldata.variables['DCHL'])

CHLA=NCHL+DCHL

obspath='/esarchive/obs/argo/bgc/climatologies/bbp700_pisces_climatology.nc'
obsdata=Dataset(obspath,'r')

latranges=[-70.,-35.,0.,35.,70.]

lats=np.array(obsdata.variables['nav_lat'])

obsmeans=np.array(obsdata.variables['all_mean'])

(T,Z,Y,X)=np.shape(obsmeans)

statistics={}
statistics['0']=['Measurement']
statistics['1']=['Sensor Type']
statistics['2']=['Depth']
statistics['3']=['Latitude']
statistics['4']=['Correlation (R)']
statistics['5']=['Standard Error']

depths={'0-100m':range(10), '100-500m':range(11,20), '500-1000m':range(21,30)}
sensortypes=['all','124deg','142-150deg'] ## for chla, only 'all'
stats=['mean','median']

pathout='/esarchive/scratch/mfalls/bgcargo'
os.chdir(pathout)

for depth in depths.keys():
    for sensor in sensortypes:
        for stat in stats:
            lists = {}
            obs=np.array(obsdata.variables[sensor+'_'+stat])

            lists['obs-70_-35'] = []
            lists[stat + '-70_-35'] = []
            lists['obs-35_0'] = []
            lists[stat + '-35_0'] = []
            lists['obs0_35'] = []
            lists[stat + '0_35'] = []
            lists['obs35_70'] = []
            lists[stat + '35_70'] = []

            for x in range(X):
                for y in range(Y):
                    if lats[y,x]<latranges[-1] and lats[y,x]>latranges[0]:
                        for t in range(T):
                            if obs[t,0,y,x]!=np.nan and obs[t,0,y,x]!=0: # and int(BBP700[t,0,y,x])!=600000012024526405632:
                                for z in depths[depth]:
                                    if not np.isnan(obs[t,z,y,x])and int(BBP700[t,z,y,x])!=600000012024526405632:
                                        if lats[y,x]<-35. and lats[y,x]>-70.:
                                            lists['obs-70_-35'].append(obs[t,z,y,x])
                                            lists[stat + '-70_-35'].append(BBP700[t,z,y,x])
                                        elif lats[y,x]<0.:
                                            lists['obs-35_0'].append(obs[t,z,y,x])
                                            lists[stat + '-35_0'].append(BBP700[t,z,y,x])
                                        elif lats[y,x]<35.:
                                            lists['obs0_35'].append(obs[t,z,y,x])
                                            lists[stat + '0_35'].append(BBP700[t,z,y,x])
                                        elif lats[y,x]<70.:
                                            lists['obs35_70'].append(obs[t,z,y,x])
                                            lists[stat + '35_70'].append(BBP700[t,z,y,x])
                                    else:
                                        break

            lists['obs-70_-35'], lists[stat+'-70_-35'] = remove_outliers(lists['obs-70_-35'], lists[stat+'-70_-35'])
            lists['obs-35_0'], lists[stat + '-35_0'] = remove_outliers(lists['obs-35_0'], lists[stat + '-35_0'])
            lists['obs0_35'], lists[stat + '0_35'] = remove_outliers(lists['obs0_35'], lists[stat + '0_35'])
            lists['obs35_70'], lists[stat + '35_70'] = remove_outliers(lists['obs35_70'], lists[stat + '35_70'])



            ##### put all statistical parameters here

            latstrings=['-70_-35','-35_0','0_35','35_70']
            regs={}
            #for i in range(4):
            for lat in latstrings:
                statistics['0'].append(stat)
                statistics['1'].append(sensor)
                statistics['2'].append(depth)
                statistics['3'].append(lat)
                slope,intercept,rvalue,pvalue,stderr=linregress(lists['obs'+lat], lists[stat + lat])
                statistics['4'].append(rvalue)
                statistics['5'].append(stderr)
                regs[lat]=[slope,intercept]

            # choosing random elements from the list to make the scatters more readable
            lists['obs-70_-35'], lists[stat+'-70_-35'] = pick_randoms(lists['obs-70_-35'], lists[stat+'-70_-35'],200)
            lists['obs-35_0'], lists[stat + '-35_0'] = pick_randoms(lists['obs-35_0'], lists[stat + '-35_0'],200)
            lists['obs0_35'], lists[stat + '0_35'] = pick_randoms(lists['obs0_35'], lists[stat + '0_35'],200)
            lists['obs35_70'], lists[stat + '35_70'] = pick_randoms(lists['obs35_70'], lists[stat + '35_70'],200)


            plt.figure(1)
            plt.suptitle(depth +' '+stat+' for '+ sensor + ' sensors')
            plt.xlabel('Observed')
            plt.ylabel('Model')

            ax = plt.subplot(221)
            ax.plot(lists['obs-70_-35'], lists[stat+'-70_-35'], 'r+',markersize=2)
            ax.set_title('-70 to -35 deg', size=6)
            ax.set_xlabel('Observed', size=6)
            ax.set_ylabel('Model', size=6)
            xticks=ax.get_xticks()
            x = np.linspace(min(lists['obs-70_-35']), max(lists['obs-70_-35']), 100)
            y=[regs['-70_-35'][0]*i+regs['-70_-35'][1] for i in x]
            ax.plot(x,y,'b-')
            ax.set_xticklabels(xticks, size=6)
            ax.set_yticklabels(ax.get_yticks(), size=6)

            ax = plt.subplot(222)
            ax.plot(lists['obs-35_0'], lists[stat + '-35_0'], 'r+',markersize=2)
            ax.set_title('-35 to 0 deg', size=6)
            ax.set_xlabel('Observed', size=6)
            ax.set_ylabel('Model', size=6)
            xticks=ax.get_xticks()
            x = np.linspace(min(lists['obs-35_0']), max(lists['obs-35_0']), 100)
            y=[regs['-35_0'][0]*i+regs['-35_0'][1] for i in x]
            ax.plot(x,y,'b-')
            ax.set_xticklabels(xticks, size=6)
            ax.set_yticklabels(ax.get_yticks(), size=6)

            ax = plt.subplot(223)
            ax.plot(lists['obs0_35'], lists[stat + '0_35'], 'r+',markersize=2)
            ax.set_title('0 to 35 deg', size=6)
            ax.set_xlabel('Observed', size=6)
            ax.set_ylabel('Model', size=6)
            xticks=ax.get_xticks()
            x = np.linspace(min(lists['obs0_35']), max(lists['obs0_35']), 100)
            y=[regs['0_35'][0]*i+regs['0_35'][1] for i in x]
            ax.plot(x,y,'b-')
            ax.set_xticklabels(xticks, size=6)
            ax.set_yticklabels(ax.get_yticks(), size=6)

            ax = plt.subplot(224)
            ax.plot(lists['obs35_70'], lists[stat + '35_70'], 'r+',markersize=2)
            ax.set_title('35 to 70 deg', size=6)
            ax.set_xlabel('Observed', size=6)
            ax.set_ylabel('Model', size=6)
            xticks=ax.get_xticks()
            x=np.linspace(min(lists['obs35_70']),max(lists['obs35_70']),100)
            y=[regs['35_70'][0]*i+regs['35_70'][1] for i in x]
            ax.plot(x,y,'b-')
            ax.set_xticklabels(xticks, size=6)
            ax.set_yticklabels(ax.get_yticks(), size=6)

            plt.savefig(stat+'_'+sensor+'_'+depth)
            print(stat+'_'+sensor+'_'+depth+' printed')
            plt.show()
            plt.clf()

statistics=DataFrame(statistics)
statistics=statistics.to_csv()
f=open("Statistical_Parameters.csv","w+")
f.write(statistics)
print("Statistical_Parameters.csv written")